from django.views.generic import ListView
from django.db.models import Q
from django.shortcuts import redirect
from django.template.defaultfilters import slugify

from snowebsvg.models import Collection, GroupSvg, Svg


class CollectionList(ListView):
    model = Collection
    template_name = 'app/collection_list.html'


class GroupSvgList(ListView):
    model = GroupSvg
    template_name = 'app/groupsvg_list.html'

    def get_queryset(self):
        collection_key = self.kwargs.get('collection_key')
        if collection_key:
            return self.model.objects.filter(collection__key=collection_key)
        return self.model.objects.all()


class SvgList(ListView):
    model = Svg
    template_name = 'app/svg_list.html'

    def get_queryset(self):
        group_key = self.kwargs.get('group_key')
        collection_key = self.kwargs.get('collection_key')
        svg_key = self.kwargs.get('svg_key')
        if group_key and collection_key and svg_key:
            return self.model.objects.filter(
                group__key=group_key,
                group__collection__key=collection_key,
                key=svg_key
            )
        return self.model.objects.all()


class SvgSearch(ListView):
    model = Svg
    template_name = 'app/svg_search.html'

    def get_queryset(self):
        key = self.kwargs.get('key')
        if key:
            return self.model.objects.filter(
                Q(key__icontains=key) |
                Q(group__key__icontains=key) |
                Q(group__collection__key__icontains=key)
            )
        return self.model.objects.all()

    def get_context_data(self, **kwargs):
        context = super(SvgSearch, self).get_context_data(**kwargs)
        context['key'] = self.kwargs.get('key')
        return context

    def post(self, request, *args, **kwargs):
        key = request.POST.get('key', None)
        if key:
            key_slug = slugify(key)
            if key_slug:
                return redirect('app:svg_search', key=key_slug)
        return redirect('app:svg_search')



import logging
from django.test import TestCase
from django.core import management

from snowebsvg.models import Svg, Collection
from snowebsvg.templatetags.svg import svg_inline, svg_django, collection_styles, svg_preview

logger = logging.getLogger('snowebsvg')


class SnowebSvgTest(TestCase):

    def test_svg_build(self):
        management.call_command('svg_build')

    def test_templatetags_svg_inline(self):
        self.test_svg_build()
        for svg in Svg.objects.all():
            logger.debug("svg_inline svg=`%s`" % str(svg))
            svg_inline(svg)
        for svg in Svg.objects.all():
            logger.debug("svg_inline key_composer=`%s`" % str(svg.key_composer))
            svg_inline(svg.key_composer)

    def test_templatetags_svg_django(self):
        self.test_svg_build()
        for svg in Svg.objects.all():
            logger.debug("svg_django svg=`%s`" % str(svg))
            svg_django(svg)
        for svg in Svg.objects.all():
            logger.debug("svg_django key_composer=`%s`" % str(svg.key_composer))
            svg_django(svg.key_composer)

    def test_templatetags_svg_preview(self):
        self.test_svg_build()
        for svg in Svg.objects.all():
            logger.debug("svg_preview svg=`%s`" % str(svg))
            svg_preview(svg)
        for svg in Svg.objects.all():
            logger.debug("svg_preview key_composer=`%s`" % str(svg.key_composer))
            svg_preview(svg.key_composer)

    def test_templatetags_collection_styles(self):
        self.test_svg_build()
        for collection in Collection.objects.all():
            logger.debug("collection_styles collection=`%s`" % str(collection))
            collection_styles(collection)



from django import template
from snowebsvg.models import Svg

register = template.Library()


@register.simple_tag
def svg_inline(svg_target, theme=None, size=None, grid=False):
    if isinstance(svg_target, Svg):
        return svg_target.render_html(theme, size, grid)
    else:
        svg = Svg()
        svg_composed = svg.key_decomposer(svg_target)
        return svg_composed.render_html(theme, size, grid)


@register.simple_tag
def svg_preview(svg_target, theme=None, size=None, grid=False):
    if isinstance(svg_target, Svg):
        return svg_target.render_preview(theme, size, grid)
    else:
        svg = Svg()
        svg_composed = svg.key_decomposer(svg_target)
        return svg_composed.render_preview(theme, size, grid)


@register.simple_tag
def svg_django(svg_target, theme=None, size=None):
    if isinstance(svg_target, Svg):
        return svg_target.render_django(theme, size)
    else:
        svg = Svg()
        svg_composed = svg.key_decomposer(svg_target)
        return svg_composed.render_django(theme, size)


@register.simple_tag
def collection_styles(collection, theme=None, size=None):
    return collection.render_styles(theme, size)



from django.contrib.sitemaps.views import sitemap
from django.urls import path
from django.conf import settings
from django.views.generic import TemplateView

from app.views import CollectionList, GroupSvgList, SvgList, SvgSearch
from app.sitemaps import StaticSitemap, CollectionSitemap, GroupSvgSitemap, SvgSitemap

app_name = 'app'

sitemaps = {
    'static': StaticSitemap,
    'collection': CollectionSitemap,
    'svg': SvgSitemap,
    'group_svg': GroupSvgSitemap,
}

urlpatterns = [
    path('', CollectionList.as_view(), name='collection'),
    path('collection/<collection_key>/', GroupSvgList.as_view(), name='group'),
    path('svg/<collection_key>/<group_key>/<svg_key>/', SvgList.as_view(), name='svg'),
    path('search/', SvgSearch.as_view(), name='svg_search'),
    path('search/<key>/', SvgSearch.as_view(), name='svg_search'),
    path('sitemap.xml', sitemap, {'sitemaps': sitemaps},
         name='django.contrib.sitemaps.views.sitemap')
]

if settings.DEBUG:
    urlpatterns += [
        path('test/', TemplateView.as_view(template_name='app/experimental.html'), name='experimental'),
    ]



from snowebsvg import settings


class ThemeMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if 'theme' not in request.session:
            request.session['theme'] = settings.SVG_DEFAULT_THEME
        if 'theme' in request.GET:
            request.session['theme'] = request.GET['theme']
        return self.get_response(request)



# Generated by Django 3.1.4 on 2021-01-01 10:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Collection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=255, verbose_name='name')),
            ],
        ),
        migrations.CreateModel(
            name='GroupSvg',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(help_text='dfgdfg', max_length=255, verbose_name='key')),
                ('collection', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='snowebsvg.collection')),
            ],
        ),
        migrations.CreateModel(
            name='Svg',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=255, verbose_name='title')),
                ('group', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='snowebsvg.groupsvg')),
            ],
        ),
    ]
